@@ -57,7 +57,7 @@ static char* message_completion(const char* text, int stat)
         sObject* messages = STRING_NEW_STACK("");
 
         while(1) {
-            object = hash_item(current, command->mMessages[0]);
+            object = uobject_item(current, command->mMessages[0]);
             if(object || current == gRootObject) break;
             current = SUOBJECT((current)).mParent;
         }
@@ -68,17 +68,17 @@ static char* message_completion(const char* text, int stat)
         if(object && TYPE(object) == T_UOBJECT) {
             int i;
             for(i=1; i<command->mMessagesNum; i++) {
-                object = hash_item(object, command->mMessages[i]);
+                object = uobject_item(object, command->mMessages[i]);
                 if(object == NULL || TYPE(object) != T_UOBJECT) break;
                 string_push_back(messages, command->mMessages[i]);
                 string_push_back(messages, "::");
             }
 
             if(object && TYPE(object) == T_UOBJECT) {
-                hash_it* it = hash_loop_begin(object);
+                uobject_it* it = uobject_loop_begin(object);
                 while(it) {
-                    char* key = hash_loop_key(it);
-                    sObject* item = hash_loop_item(it);
+                    char* key = uobject_loop_key(it);
+                    sObject* item = uobject_loop_item(it);
 
                     sObject* candidate = STRING_NEW_STACK(string_c_str(messages));
                     string_push_back(candidate, key);
@@ -96,7 +96,7 @@ static char* message_completion(const char* text, int stat)
                         vector_add(gCompletionArray, string_c_str(candidate));
                     }
 
-                    it = hash_loop_next(it);
+                    it = uobject_loop_next(it);
                 }
 
                 vector_sort(gCompletionArray, name_sort);
@@ -107,7 +107,7 @@ static char* message_completion(const char* text, int stat)
         index = 0;
     }
 
-    while(index < vector_size(gCompletionArray)) {
+    while(index < vector_count(gCompletionArray)) {
         char* candidate = vector_item(gCompletionArray, index);
         index++;
 
@@ -136,21 +136,21 @@ static char* all_program_completion(const char* text, int stat)
 
         sObject* current = gReadlineCurrentObject;
         while(1) {
-            hash_it* it = hash_loop_begin(current);
+            uobject_it* it = uobject_loop_begin(current);
             while(it) {
-                char* key = hash_loop_key(it);
-                sObject* object = hash_loop_item(it);
+                char* key = uobject_loop_key(it);
+                sObject* object = uobject_loop_item(it);
 
                 if(hash_item(hash, key) == NULL) {
                     hash_put(hash, key, object);
 
-                    sObject* candidate = STRING_NEW_STACK(hash_loop_key(it));
+                    sObject* candidate = STRING_NEW_STACK(uobject_loop_key(it));
                     if(TYPE(object) == T_UOBJECT) {
-                        if(hash_item(object, "main")) {
+                        if(uobject_item(object, "main")) {
                             vector_add(gCompletionArray, string_c_str(candidate));
                         }
                         else {
-                            sObject* candidate2 = STRING_NEW_STACK(hash_loop_key(it));
+                            sObject* candidate2 = STRING_NEW_STACK(uobject_loop_key(it));
                             string_push_back(candidate2, "::");
                             vector_add(gCompletionArray, string_c_str(candidate2));
                         }
@@ -160,7 +160,7 @@ static char* all_program_completion(const char* text, int stat)
                     }
                 }
 
-                it = hash_loop_next(it);
+                it = uobject_loop_next(it);
             }
             
             if(current == gRootObject) break;
@@ -173,7 +173,7 @@ static char* all_program_completion(const char* text, int stat)
         index = 0;
     }
 
-    while(index < vector_size(gCompletionArray)) {
+    while(index < vector_count(gCompletionArray)) {
         char* candidate = vector_item(gCompletionArray, index);
         index++;
 
@@ -193,21 +193,21 @@ static char* program_completion(const char* text, int stat)
 
         sObject* current = gReadlineCurrentObject;
         while(1) {
-            hash_it* it = hash_loop_begin(current);
+            uobject_it* it = uobject_loop_begin(current);
             while(it) {
-                char* key = hash_loop_key(it);
-                sObject* object = hash_loop_item(it);
+                char* key = uobject_loop_key(it);
+                sObject* object = uobject_loop_item(it);
 
                 if(hash_item(hash, key) == NULL) {
                     hash_put(hash, key, object);
 
-                    sObject* candidate = STRING_NEW_STACK(hash_loop_key(it));
+                    sObject* candidate = STRING_NEW_STACK(uobject_loop_key(it));
                     if(TYPE(object) == T_UOBJECT) {
-                        if(hash_item(object, "main")) {
+                        if(uobject_item(object, "main")) {
                             vector_add(gCompletionArray, string_c_str(candidate));
                         }
                         else {
-                            sObject* candidate2 = STRING_NEW_STACK(hash_loop_key(it));
+                            sObject* candidate2 = STRING_NEW_STACK(uobject_loop_key(it));
                             string_push_back(candidate2, "::");
                             vector_add(gCompletionArray, string_c_str(candidate2));
                         }
@@ -217,7 +217,7 @@ static char* program_completion(const char* text, int stat)
                     }
                 }
 
-                it = hash_loop_next(it);
+                it = uobject_loop_next(it);
             }
             
             if(current == gRootObject) break;
@@ -230,7 +230,7 @@ static char* program_completion(const char* text, int stat)
         index = 0;
     }
 
-    while(index < vector_size(gCompletionArray)) {
+    while(index < vector_count(gCompletionArray)) {
         char* candidate = vector_item(gCompletionArray, index);
         index++;
 
@@ -258,7 +258,7 @@ static char* user_completion(const char* text, int stat)
         gCompletionArray = VECTOR_NEW_STACK(16);
 
         int i;
-        for(i=0; i<vector_size(SFD(gUserCompletionNextout).fdbuf.mLines); i++) {
+        for(i=0; i<vector_count(SFD(gUserCompletionNextout).fdbuf.mLines); i++) {
             sObject* candidate = STRING_NEW_STACK(vector_item(SFD(gUserCompletionNextout).fdbuf.mLines, i));
             string_chomp(candidate);
             vector_add(gCompletionArray, string_c_str(candidate));
@@ -270,7 +270,7 @@ static char* user_completion(const char* text, int stat)
         index = 0;
     }
 
-    while(index < vector_size(gCompletionArray)) {
+    while(index < vector_count(gCompletionArray)) {
         char* candidate = vector_item(gCompletionArray, index);
         index++;
 
@@ -299,7 +299,7 @@ static char* redirect_completion(const char* text, int stat)
         index = 0;
     }
 
-    while(index < vector_size(gCompletionArray)) {
+    while(index < vector_count(gCompletionArray)) {
         char* candidate = string_c_str((sObject*)vector_item(gCompletionArray, index));
         index++;
 
@@ -328,23 +328,23 @@ static char* env_completion(const char* text, int stat_)
 
         sObject* current = gReadlineCurrentObject;
         while(1) {
-            hash_it* it = hash_loop_begin(current);
+            uobject_it* it = uobject_loop_begin(current);
             while(it) {
-                char* key = hash_loop_key(it);
-                sObject* object = hash_loop_item(it);
+                char* key = uobject_loop_key(it);
+                sObject* object = uobject_loop_item(it);
 
                 if(hash_item(hash, key) == NULL) {
                     if(TYPE(object) == T_STRING || TYPE(object) == T_VECTOR || TYPE(object) == T_HASH) {
                         hash_put(hash, key, object);
 
                         sObject* candidate = STRING_NEW_STACK("");
-                        string_push_back(candidate, hash_loop_key(it));
+                        string_push_back(candidate, uobject_loop_key(it));
 
                         vector_add(gCompletionArray, string_c_str(candidate));
                     }
                 }
 
-                it = hash_loop_next(it);
+                it = uobject_loop_next(it);
             }
             
             if(current == gRootObject) break;
@@ -383,7 +383,7 @@ static char* env_completion(const char* text, int stat_)
         index = 0;
     }
 
-    while(index < vector_size(gCompletionArray)) {
+    while(index < vector_count(gCompletionArray)) {
         char* candidate = vector_item(gCompletionArray, index);
         index++;
 
@@ -407,7 +407,7 @@ static void get_current_completion_object(sObject** completion_object, sObject**
     if(*current_object != gRootObject) {
         sObject* parent_object = SUOBJECT(*current_object).mParent;
         get_current_completion_object(completion_object, &parent_object);
-        if(*completion_object) *completion_object = hash_item(*completion_object, SUOBJECT(*current_object).mName);
+        if(*completion_object) *completion_object = uobject_item(*completion_object, SUOBJECT(*current_object).mName);
     }
 }
 
@@ -416,7 +416,7 @@ static sObject* access_object_compl(char* name, sObject** current)
     sObject* object;
 
     while(1) {
-        object = hash_item(*current, name);
+        object = uobject_item(*current, name);
 
         if(object || *current == gCompletionObject) { return object; }
 
@@ -511,7 +511,7 @@ char** readline_on_complete(const char* text, int start, int end)
                     int i;
                     for(i=1; i<command->mMessagesNum; i++) {
                         if(object && TYPE(object) == T_UOBJECT) {
-                            object = hash_item(object, command->mMessages[i]);
+                            object = uobject_item(object, command->mMessages[i]);
                         }
                         else {
                             break;
@@ -519,10 +519,10 @@ char** readline_on_complete(const char* text, int start, int end)
                     }
 
                     if(object && TYPE(object) == T_UOBJECT) {
-                        ucompletion = hash_item(object, command->mArgs[0]);
+                        ucompletion = uobject_item(object, command->mArgs[0]);
 
                         if(ucompletion == NULL) {
-                            ucompletion = hash_item(object, "__all__");;
+                            ucompletion = uobject_item(object, "__all__");;
                         }
                     }
                     else {
@@ -599,20 +599,20 @@ char** readline_on_complete(const char* text, int start, int end)
                     vector_add(argv, STRING_NEW_GC("", FALSE));
                     vector_add(argv, STRING_NEW_GC("", FALSE));
                 }
-                hash_put(SFUN(fun).mLocalObjects, "ARGV", argv);
+                uobject_put(SFUN(fun).mLocalObjects, "ARGV", argv);
 
                 int rcode = 0;
                 xyzsh_set_signal();
-                if(!run(ucompletion, nextin, nextout, &rcode, gReadlineCurrentObject, fun)) {
+                if(!run(SCOMPLETION(ucompletion).mBlock, nextin, nextout, &rcode, gReadlineCurrentObject, fun)) {
                     readline_signal();
                     fprintf(stderr, "\nrun time error\n");
                     fprintf(stderr, "%s", string_c_str(gErrMsg));
-                    vector_pop_back(gStackFrames);
+                    (void)vector_pop_back(gStackFrames);
                     stack_end_stack();
                     return NULL;
                 }
+                (void)vector_pop_back(gStackFrames);
                 readline_signal();
-                vector_pop_back(gStackFrames);
 
                 fd_split(nextout, kLF);
 
@@ -649,10 +649,10 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
                     p+=2;
 
-                    sObject* object2 = hash_item(object, string_c_str(str));
+                    sObject* object2 = uobject_item(object, string_c_str(str));
                     if(object2 == NULL || TYPE(object2) !=T_UOBJECT) {
                         sObject* new_object = UOBJECT_NEW_GC(8, object, string_c_str(str), TRUE);
-                        hash_put(object, string_c_str(str), new_object);
+                        uobject_put(object, string_c_str(str), new_object);
                         uobject_init(new_object);
 
                         object = new_object;
@@ -668,7 +668,7 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
             if(object && TYPE(object) == T_UOBJECT && string_c_str(str)[0] != 0) {
-                hash_put(object, string_c_str(str), block_clone_gc(block, T_COMPLETION, FALSE));
+                uobject_put(object, string_c_str(str), COMPLETION_NEW_GC(block, FALSE));
             }
             else {
                 err_msg("There is no object", runinfo->mSName, runinfo->mSLine, command->mArgsRuntime[i]);
@@ -818,9 +818,9 @@ static int readline_macro(int count, int key)
                 fprintf(stderr, "%s", string_c_str(gErrMsg));
             }
         }
+        (void)vector_pop_back(gStackFrames);
         readline_signal();
         //xyzsh_restore_signal_default();
-        (void)vector_pop_back(gStackFrames);
     }
     else {
         fprintf(stderr, "parser error\n");
