@@ -39,7 +39,7 @@ void run_init(enum eAppType app_type)
     gStdout = FD_NEW_STACK(kFDKindBuf, 0);
     gStderr = FD_NEW_STACK(kFDKindBuf, 0);
     gJobs = VECTOR_NEW_GC(16, FALSE);
-    hash_put(gxyzshObject, "_jobs", gJobs);
+    uobject_put(gxyzshObject, "_jobs", gJobs);
 
     xyzsh_job_done = NULL;
 
@@ -70,6 +70,7 @@ sObject* fd_new_from_stack(int kind, int fd)
         SFD(self).fdbuf.mBufLen = 0;
 
         SFD(self).fdbuf.mLines = VECTOR_NEW_MALLOC(32);
+        SFD(self).fdbuf.mReadedLineNumber = 0;
     }
 
     return self;
@@ -80,7 +81,7 @@ void fd_delete_stack(sObject* self)
     if(SFD(self).mKind == kFDKindBuf) {
         sObject* v = SFD(self).fdbuf.mLines;
         int i;
-        for(i=0; i<vector_size(v); i++) {
+        for(i=0; i<vector_count(v); i++) {
             FREE(vector_item(v, i));
         }
         vector_delete_malloc(SFD(self).fdbuf.mLines);
@@ -123,14 +124,6 @@ BOOL bufsiz_write(int fd, char* buf, int buf_size)
     return TRUE;
 }
 
-/*
-// TRUE: Success
-// FALSE: signal interrupt. set RCODE_SIGNAL_INTERRUPT on rcode and err message
-BOOL fd_read(sObject* self)
-{
-}
-*/
-
 void fd_split(sObject* self, enum eLineField lf)
 {
     assert(TYPE(self) == T_FD && SFD(self).mKind == kFDKindBuf);
@@ -139,13 +132,14 @@ void fd_split(sObject* self, enum eLineField lf)
 
     if(SFD(self).fdbuf.mLinesLineField != lf) {
         int i;
-        for(i=0; i<vector_size(v); i++) {
+        for(i=0; i<vector_count(v); i++) {
             FREE(vector_item(v, i));
         }
         vector_clear(v);
+        SFD(self).fdbuf.mReadedLineNumber = 0;
     }
 
-    if(vector_size(v) == 0) {
+    if(vector_count(v) == 0) {
         char* p = SFD(self).fdbuf.mBuf;
         char* new_line = p;
         if(lf == kCRLF) {
@@ -253,10 +247,11 @@ void fd_clear(sObject* self)
         SFD(self).fdbuf.mBuf[0] = 0;
 
         int i;
-        for(i=0; i<vector_size(SFD(self).fdbuf.mLines); i++) {
+        for(i=0; i<vector_count(SFD(self).fdbuf.mLines); i++) {
             FREE(vector_item(SFD(self).fdbuf.mLines, i));
         }
         vector_clear(SFD(self).fdbuf.mLines);
+        SFD(self).fdbuf.mReadedLineNumber = 0;
     }
 }
 
@@ -816,7 +811,7 @@ BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* run
     for(i=0; i<argc; i++) {
         vector_add(argv2, STRING_NEW_GC(argv[i], FALSE));
     }
-    hash_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
+    uobject_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
 
     sCommand* command = runinfo->mCommand;
     sObject* options = HASH_NEW_GC(16, FALSE);
@@ -830,7 +825,7 @@ BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* run
             }
         }
     }
-    hash_put(SFUN(fun).mLocalObjects, "OPTIONS", options);
+    uobject_put(SFUN(fun).mLocalObjects, "OPTIONS", options);
 
     sObject* arg_blocks = SFUN(fun).mArgBlocks;
     SFUN(fun).mArgBlocks = VECTOR_NEW_GC(16, FALSE);
@@ -842,7 +837,8 @@ BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* run
     }
 
     int rcode = 0;
-    if(!run(fun, nextin, nextout, &rcode, runinfo->mCurrentObject, fun)) {
+
+    if(!run(SFUN(fun).mBlock, nextin, nextout, &rcode, runinfo->mCurrentObject, fun)) {
         if(rcode != RCODE_RETURN) {
             char buf[BUFSIZ];
             err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -853,10 +849,10 @@ BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* run
         }
     }
 
-runinfo->mRCode = rcode;
-
     (void)vector_pop_back(gStackFrames);
 
+runinfo->mRCode = rcode;
+
     SFUN(fun).mLocalObjects = local_objects;
     SFUN(fun).mArgBlocks = arg_blocks;
     SFUN(fun).mRunInfo = runinfo2;
@@ -918,8 +914,8 @@ static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunI
             }
             break;
 
-        case T_EXTERNAL:
-            if(!run_external_command(SEXTERNAL(object).mPath, nextin, nextout, runinfo)) {
+        case T_EXTPROG:
+            if(!run_external_command(SEXTPROG(object).mPath, nextin, nextout, runinfo)) {
                 return FALSE;
             }
             break;
@@ -959,7 +955,7 @@ static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunI
             break;
 
         case T_UOBJECT: {
-            sObject* main_obj = hash_item(object, "main");
+            sObject* main_obj = uobject_item(object, "main");
             runinfo->mRecieverObject = object;
 
             if(main_obj) {
@@ -977,7 +973,7 @@ static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunI
 
         case T_VECTOR: {
             int j;
-            for(j=0; j<vector_size(object); j++) {
+            for(j=0; j<vector_count(object); j++) {
                 sObject* item = vector_item(object, j);
 
                 if(!run_object(item, nextin, nextout, runinfo)) {
@@ -1078,17 +1074,17 @@ static BOOL redirect_ready(sObject** nextout, sObject** nextin, int* opened_fd,
                 }
 
                 SFD(*nextin).mKind = kFDKindBuf;
-                fd = open(command->mRedirectsFileNames[i], O_RDONLY);
+                fd = open(command->mRedirectsFileNamesRuntime[i], O_RDONLY);
                 if(fd < 0) {
                     char buf[BUFSIZ];
-                    snprintf(buf, BUFSIZ, "redirect: error with openning %s file", command->mRedirectsFileNames[i]);
+                    snprintf(buf, BUFSIZ, "redirect: error with openning %s file", command->mRedirectsFileNamesRuntime[i]);
                     err_msg(buf, runinfo->mSName, runinfo->mSLine, "");
                     return FALSE;
                 }
 
                 if(!bufsiz_read(fd, ALLOC &result, &result_len, &result_size)) {
                     char buf[BUFSIZ];
-                    snprintf(buf, BUFSIZ, "redirect: error with reading %s file", command->mRedirectsFileNames[i]);
+                    snprintf(buf, BUFSIZ, "redirect: error with reading %s file", command->mRedirectsFileNamesRuntime[i]);
                     err_msg(buf, runinfo->mSName, runinfo->mSLine, "");
                     return FALSE;
                 }
@@ -1101,11 +1097,11 @@ static BOOL redirect_ready(sObject** nextout, sObject** nextin, int* opened_fd,
                 break;
 
             case REDIRECT_APPEND:
-                if(SFD(*nextout).mKind == kFDKindBuf) {
-                    fd = open(command->mRedirectsFileNames[i], O_WRONLY|O_APPEND|O_CREAT, 0644);
+                if(*nextout == NULL) {
+                    fd = open(command->mRedirectsFileNamesRuntime[i], O_WRONLY|O_APPEND|O_CREAT, 0644);
                     if(fd < 0) {
                         char buf[BUFSIZ];
-                        snprintf(buf, BUFSIZ, "redirect: error with openning %s file", command->mRedirectsFileNames[i]);
+                        snprintf(buf, BUFSIZ, "redirect: error with openning %s file", command->mRedirectsFileNamesRuntime[i]);
                         err_msg(buf, runinfo->mSName, runinfo->mSLine, "");
                         return FALSE;
                     }
@@ -1116,11 +1112,11 @@ static BOOL redirect_ready(sObject** nextout, sObject** nextin, int* opened_fd,
                 break;
 
             case REDIRECT_OUT:
-                if(SFD(*nextout).mKind == kFDKindBuf) {
-                    fd = open(command->mRedirectsFileNames[i], O_WRONLY|O_CREAT|O_TRUNC, 0644);
+                if(*nextout == NULL) {
+                    fd = open(command->mRedirectsFileNamesRuntime[i], O_WRONLY|O_CREAT|O_TRUNC, 0644);
                     if(fd < 0) {
                         char buf[BUFSIZ];
-                        snprintf(buf, BUFSIZ, "redirect: error with openning %s file", command->mRedirectsFileNames[i]);
+                        snprintf(buf, BUFSIZ, "redirect: error with openning %s file", command->mRedirectsFileNamesRuntime[i]);
                         err_msg(buf, runinfo->mSName, runinfo->mSLine, "");
                         return FALSE;
                     }
@@ -1144,6 +1140,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
 {
     sObject* nextin;
     if(statment->mFlags & STATMENT_CONTEXTPIPE) {
+        /// read stdin ///
         if(pipein == gStdin && SFD(gStdin).fdbuf.mBufLen == 0) {
             if(!stdin_read_out()) {
                 sCommand* command = runinfo->mCommand;
@@ -1158,7 +1155,36 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
             }
         }
 
-        nextin = pipein;
+        short line_number = statment->mFlags & STATMENT_CONTEXTPIPE_NUMBER;
+
+        /// context pipe ///
+        if(line_number == 0) {
+            nextin = pipein;
+        }
+        /// line context pipe ///
+        else {
+            nextin = FD_NEW_STACK(kFDKindBuf, 0);
+            fd_split(pipein, kLF);
+            if(line_number < 0) line_number += vector_count(SFD(pipein).fdbuf.mLines)+1;
+            if(line_number < 0) line_number = 0;
+            if(line_number > vector_count(SFD(pipein).fdbuf.mLines)) line_number = vector_count(SFD(pipein).fdbuf.mLines);
+            int i;
+            for(i=SFD(pipein).fdbuf.mReadedLineNumber; i<vector_count(SFD(pipein).fdbuf.mLines) && i-SFD(pipein).fdbuf.mReadedLineNumber<line_number; i++) {
+                char* line = vector_item(SFD(pipein).fdbuf.mLines, i);
+                if(!fd_write(nextin, line, strlen(line))) {
+                    sCommand* command = runinfo->mCommand;
+                    if(command) {
+                        err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                    }
+                    else {
+                        err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine, "");
+                    }
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+            }
+            SFD(pipein).fdbuf.mReadedLineNumber += line_number;
+        }
     }
     else if(statment->mFlags & STATMENT_GLOBALPIPEIN) {
         nextin = FD_NEW_STACK(kFDKindBuf, 0);
@@ -1186,30 +1212,33 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         const BOOL last_program = runinfo->mLastProgram = i == statment->mCommandsNum-1;
         runinfo->mFilter = i != 0 || (statment->mFlags & (STATMENT_CONTEXTPIPE|STATMENT_GLOBALPIPEIN));
 
-        sObject* nextout;
-        if(last_program) {
-            if(statment->mFlags & STATMENT_GLOBALPIPEOUT) {
-                fd_clear(gGlobalPipe);
-                nextout = gGlobalPipe;
-            }
-            else {
-                nextout = pipeout;
-            }
-        }
-        else {
-            nextout = FD_NEW_STACK(kFDKindBuf, 0);
-        }
-
         /// expand env ///
-        if(!sCommand_expand_env_redirect(command, nextin, nextout, runinfo))
+        if(!sCommand_expand_env_redirect(command, nextin, runinfo))
         {
             return FALSE;
         }
 
+        sObject* nextout = NULL;
+
         int opened_fd = -1;
         if(!redirect_ready(&nextout, &nextin, &opened_fd, pipeout, last_program, statment, command, runinfo)) {
             return TRUE;
         }
+        
+        if(nextout == NULL) {
+            if(last_program) {
+                if(statment->mFlags & STATMENT_GLOBALPIPEOUT) {
+                    fd_clear(gGlobalPipe);
+                    nextout = gGlobalPipe;
+                }
+                else {
+                    nextout = pipeout;
+                }
+            }
+            else {
+                nextout = FD_NEW_STACK(kFDKindBuf, 0);
+            }
+        }
 
         if(command->mArgsNum > 0) {
             if(command->mArgsEnv[0]) {
@@ -1225,7 +1254,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                 while(i < command->mMessagesNum) {
                     if(object && TYPE(object) == T_UOBJECT) {
                         reciever = object;
-                        object = hash_item(object, command->mMessages[i++]);
+                        object = uobject_item(object, command->mMessages[i++]);
                     }
                     else {
                         err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -1235,7 +1264,46 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
 
                 if(object && TYPE(object) == T_UOBJECT) {
                     reciever = object;
-                    object = hash_item(object, command->mArgs[0]);
+                    object = uobject_item(object, command->mArgs[0]);
+
+                    if(object == gInherit) {
+                        sObject* running_object = runinfo->mRunningObject;
+
+                        if(runinfo->mRunningObject) {
+                            switch(TYPE(running_object)) {
+                            case T_FUN:  {
+                                sObject* parent = SFUN(running_object).mParent;
+
+                                if(parent) {
+                                    if(TYPE(parent) == T_NFUN || TYPE(parent) == T_FUN) {
+                                        object = parent;
+                                    }
+                                }
+                                }
+                                break;
+
+                            case T_CLASS: {
+                                sObject* parent = SFUN(running_object).mParent;
+
+                                if(parent) {
+                                    if(TYPE(parent) == T_CLASS) {
+                                        object = parent;
+                                    }
+                                }
+                                }
+                                break;
+
+                            case T_NFUN:
+                            case T_BLOCK:
+                                err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                                return FALSE;
+
+                            default:
+                                fprintf(stderr, "unexpected error in cmd_inherit\n");
+                                exit(1);
+                            }
+                        }
+                    }
 
                     /// expand env ///
                     if(!sCommand_expand_env(command, object, nextin, nextout, runinfo))
@@ -1264,6 +1332,45 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                 sObject* reciever = current_object;
                 sObject* object = access_object(command->mArgs[0], &reciever, runinfo->mRunningObject);
 
+                if(object == gInherit) {
+                    sObject* running_object = runinfo->mRunningObject;
+
+                    if(runinfo->mRunningObject) {
+                        switch(TYPE(running_object)) {
+                        case T_FUN:  {
+                            sObject* parent = SFUN(running_object).mParent;
+
+                            if(parent) {
+                                if(TYPE(parent) == T_NFUN || TYPE(parent) == T_FUN) {
+                                    object = parent;
+                                }
+                            }
+                            }
+                            break;
+
+                        case T_CLASS: {
+                            sObject* parent = SFUN(running_object).mParent;
+
+                            if(parent) {
+                                if(TYPE(parent) == T_CLASS) {
+                                    object = parent;
+                                }
+                            }
+                            }
+                            break;
+
+                        case T_NFUN:
+                        case T_BLOCK:
+                            err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                            return FALSE;
+
+                        default:
+                            fprintf(stderr, "unexpected error in cmd_inherit\n");
+                            exit(1);
+                        }
+                    }
+                }
+
                 if(object) {
                     /// expand env ///
                     if(!sCommand_expand_env(command, object, nextin, nextout, runinfo))
@@ -1424,7 +1531,7 @@ if(!(n++ % 32)) (void)gc_sweep();
 
 BOOL forground_job(int num)
 {
-    if(num>=0 && num < vector_size(gJobs)) {
+    if(num>=0 && num < vector_count(gJobs)) {
         sObject* job = vector_item(gJobs, num);
 
         tcsetattr(STDIN_FILENO, TCSANOW, SJOB(job).mTty);
@@ -1492,7 +1599,7 @@ BOOL forground_job(int num)
 
 BOOL background_job(int num)
 {
-    if(num>=0 && num < vector_size(gJobs)) {
+    if(num>=0 && num < vector_count(gJobs)) {
         sObject* job = vector_item(gJobs, num);
         
         if(kill(SJOB(job).mPGroup, SIGCONT) < 0) {
@@ -1511,7 +1618,7 @@ void xyzsh_wait_background_job()
 
     if(pid >= 0) {
         int i;
-        for(i=0; i<vector_size(gJobs); i++) {
+        for(i=0; i<vector_count(gJobs); i++) {
             sObject* job = vector_item(gJobs, i);
 
             if(pid == SJOB(job).mPGroup) {
@@ -1527,7 +1634,7 @@ void xyzsh_wait_background_job()
 void xyzsh_kill_all_jobs()
 {
     int i;
-    for(i=0; i<vector_size(gJobs); i++) {
+    for(i=0; i<vector_count(gJobs); i++) {
         sObject* job = vector_item(gJobs, i);
 
         kill(SJOB(job).mPGroup, SIGKILL);
@@ -1617,29 +1724,30 @@ BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo
         for(i=0; i<argc; i++) {
             vector_add(argv2, STRING_NEW_GC(argv[i], FALSE));
         }
-        hash_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
+        uobject_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
 
         if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, fun))
         {
             if(rcode == RCODE_EXIT) {
             }
             else {
-                stack_end_stack();
+                runinfo->mRCode = rcode;
+
                 (void)vector_pop_back(gStackFrames);
+                stack_end_stack();
                 return FALSE;
             }
         }
+        (void)vector_pop_back(gStackFrames);
     }
     else {
         FREE(buf);
         sCommand* command = runinfo->mCommand;
         err_msg_adding("parser error", runinfo->mSName, runinfo->mSLine, "parser");
         stack_end_stack();
-        (void)vector_pop_back(gStackFrames);
         return FALSE;
     }
 
-    (void)vector_pop_back(gStackFrames);
     stack_end_stack();
 
     return TRUE;
