@@ -107,11 +107,14 @@ static void xyzsh_read_rc()
     xyzsh_read_rc_core(user_rc_path);
 
 
-//    hash_put(gRootObject, "test", sTestObj_new("AAA", 1));
+//    uobject_put(gRootObject, "test", sTestObj_new("AAA", 1));
 }
 
 void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
 {
+    setenv("XYZSH_VERSION", "1.1.1", 1);
+    setenv("XYZSH_DATAROOTDIR", DATAROOTDIR, 1);
+
     stack_init(1);;
     stack_start_stack();
 
@@ -129,7 +132,7 @@ void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
     xyzsh_readline_init(FALSE);
 
     gDirStack = VECTOR_NEW_GC(10, FALSE);
-    hash_put(gxyzshObject, "_dir_stack", gDirStack);
+    uobject_put(gxyzshObject, "_dir_stack", gDirStack);
 
     char* term_env = getenv("TERM");
     if(term_env != NULL && strcmp(term_env, "") != 0) {
